@@ -113,6 +113,9 @@ class RstFile(object):
         self.append(self.headings[depth] * len(text))
         self.append()
 
+    def a(self, href, text):
+        self.append('`%s <%s>`_' % (text, href))
+
     def start_bloc(self, indent=4):
         self.current._start_bloc(indent)
 
@@ -373,6 +376,8 @@ class SphinxWriter(object):
     def add_text(self, text, font_style=None):
         if font_style == 'italic':
             text = '*' + text + '*'
+        elif font_style == 'underline':
+            text = '*' + text + '*'
         elif font_style == 'bold':
             text = '**' + text + '**'
         elif font_style == 'fixed':
@@ -409,6 +414,9 @@ class SphinxWriter(object):
     def h6(self, text):
         self.current_file.heading(text, 5)
 
+    def a(self, href, text):
+        self.current_file.a(href, text)
+
     def p_start(self):
         #self.current_file.start_bloc()
         pass
@@ -420,13 +428,16 @@ class SphinxWriter(object):
     def directive(self, *args, **kw):
         self.current_file.directive(*args, **kw)
 
-    def list_item(self, text=None):
-        self.current_file.start_line('* ')
+    def list_item(self, text=None, numbered=False):
+        if numbered:
+            self.current_file.start_line('#.  ')
+        else:
+            self.current_file.start_line('*   ')
         if text is not None:
             self.current_file.append(text)
             self.current_file.append()
         else:
-            self.current_file.start_bloc(2)
+            self.current_file.start_bloc(4)
 
     def end_bloc(self):
         self.current_file.end_bloc()
@@ -529,6 +540,7 @@ class ODTReader(object):
         self.visitor = visitor
 
         self._load_styles()
+        self._load_list_styles()
 
         self.__it = None
         self.done = False
@@ -543,6 +555,22 @@ class ODTReader(object):
             open(fname, 'w').write(buffer)
         return root
 
+    def _load_list_styles(self):
+        self.list_styles = {None: None}
+        stylenodes = self.xml_styles.findall(
+                './/{%(text)s}list-style' % self.ns)
+        stylenodes.extend(
+            self.xml_content.findall('.//{%(text)s}list-style' % self.ns))
+        for node in stylenodes:
+            name = node.get('{%(style)s}name' % self.ns)
+            props = {}
+            first_child = node[0]
+            if first_child.tag == '{%(text)s}list-level-style-number' % self.ns:
+                props['numbered'] = True
+            elif first_child.tag == '{%(text)s}list-level-style-bullet' % self.ns:
+                props['numbered'] = False
+            self.list_styles[name] = props
+
     def _load_styles(self):
         self.styles = {None: None}
         stylenodes = self.xml_styles.findall('.//{%(style)s}style' % self.ns)
@@ -559,8 +587,15 @@ class ODTReader(object):
             for child in node.getchildren():
                 if child.tag == '{%(style)s}text-properties' % self.ns:
                     font_style = child.get('{%(fo)s}font-style' % self.ns)
-                    if font_style in ('italic', 'bold'):
-                        style['font-style'] = font_style
+                    if font_style == 'italic':
+                        style['font-style'] = 'italic'
+                    font_weight = child.get('{%(fo)s}font-weight' % self.ns)
+                    if font_weight == 'bold':
+                        style['font-style'] = 'bold'
+                    underline_style = child.get(
+                        '{%(style)s}text-underline-style' % self.ns)
+                    if underline_style == 'solid':
+                        style['font-style'] = 'underline'
 
             # Special case. We know that the "Source_xx_Text" styles
             # are for source code.
@@ -658,6 +693,11 @@ class ODTReader(object):
             fcontent = self.archive.read(href)
         return fname, fcontent
 
+    def is_list_numbered(self, node):
+        style_name = node.get('{%(text)s}style-name' % self.ns)
+        style = self.list_styles[style_name]
+        return style['numbered']
+
     def handle_node(self, node):
         if node.tag in [
                 '{%(text)s}span' % self.ns,
@@ -688,7 +728,7 @@ class ODTReader(object):
                 for h in headings:
                     self.handle_node(h)
             else:
-                self.start_visit('list')
+                self.start_visit('list', numbered=self.is_list_numbered(node))
                 self.recurse_node(node)
                 self.end_visit('list')
         if node.tag == '{%(text)s}list-item' % self.ns:
@@ -740,6 +780,12 @@ class ODTReader(object):
         if node.tag == '{%(table)s}covered-table-cell' % self.ns:
             self.call_visitor('covered_table_cell')
 
+        if node.tag == '{%(text)s}a' % self.ns:
+            href = node.get('{%(xlink)s}href' % self.ns)
+            self.start_visit('a', href)
+            self.recurse_node(node)
+            self.end_visit('a')
+
     def iterate(self):
         nodes = self.xml_content.findall(
             '{%(office)s}body/{%(office)s}text/*' % self.ns)
@@ -770,6 +816,7 @@ class Converter(object):
         #self.done = False
         self.buffer = None
         self.last_bloc = None
+        self.a_href = None
 
     def option(self, name):
         return self.options.get(name, '')
@@ -807,6 +854,15 @@ class Converter(object):
     def on_p_end(self):
         self.writer.p_end()
 
+    def on_a_start(self, href):
+        assert self.a_href is None, "Nested anchors are not supported"
+        self.a_href = href
+        self._start_buffering()
+
+    def on_a_end(self):
+        self.writer.a(self.a_href, self._get_buffer())
+        self.a_href = None
+
     def on_note_start(self):
         self.hightlight_start('note')
 
@@ -874,14 +930,14 @@ class Converter(object):
     def on_h6_end(self):
         self.writer.h6(self._get_buffer())
 
-    def on_list_start(self):
-        pass
+    def on_list_start(self, numbered):
+        self.numbered_list = numbered
 
     def on_list_end(self):
         pass
 
     def on_list_item_start(self):
-        self.writer.list_item()
+        self.writer.list_item(numbered=self.numbered_list)
 
     def on_list_item_end(self):
         self.writer.end_bloc()
